@@ -1,8 +1,40 @@
 import pickle
 import numpy as np
+import matplotlib.pyplot as plt
 from .obs import Obs
 
 
+def errorbar(x, y, axes=plt, **kwargs):
+    """pyerrors wrapper for the errorbars method of matplotlib
+
+    Parameters
+    ----------
+    x : list
+        A list of x-values which can be Obs.
+    y : list
+        A list of y-values which can be Obs.
+    axes : (matplotlib.pyplot.axes)
+        The axes to plot on. default is plt.
+    """
+    val = {}
+    err = {}
+    for name, comp in zip(["x", "y"], [x, y]):
+        if all(isinstance(o, Obs) for o in comp):
+            if not all(hasattr(o, 'e_dvalue') for o in comp):
+                [o.gamma_method() for o in comp]
+            val[name] = [o.value for o in comp]
+            err[name] = [o.dvalue for o in comp]
+        else:
+            val[name] = comp
+            err[name] = None
+
+        if f"{name}err" in kwargs:
+            err[name] = kwargs.get(f"{name}err")
+            kwargs.pop(f"{name}err", None)
+
+    axes.errorbar(val["x"], val["y"], xerr=err["x"], yerr=err["y"], **kwargs)
+
+
 def dump_object(obj, name, **kwargs):
     """Dump object into pickle file.
 
@@ -14,6 +46,10 @@ def dump_object(obj, name, **kwargs):
         name of the file
     path : str
         specifies a custom path for the file (default '.')
+
+    Returns
+    -------
+    None
     """
     if 'path' in kwargs:
         file_name = kwargs.get('path') + '/' + name + '.p'
@@ -30,6 +66,11 @@ def load_object(path):
     ----------
     path : str
         path to the file
+
+    Returns
+    -------
+    object : Obs
+        Loaded Object
     """
     with open(path, 'rb') as file:
         return pickle.load(file)
@@ -48,6 +89,11 @@ def pseudo_Obs(value, dvalue, name, samples=1000):
         name of the ensemble for which the Obs is to be generated.
     samples: int
         number of samples for the Obs (default 1000).
+
+    Returns
+    -------
+    res : Obs
+        Generated Observable
     """
     if dvalue <= 0.0:
         return Obs([np.zeros(samples) + value], [name])
@@ -83,6 +129,11 @@ def gen_correlated_data(means, cov, name, tau=0.5, samples=1000):
         every dataset.
     samples : int
         number of samples to be generated for each observable.
+
+    Returns
+    -------
+    corr_obs : list[Obs]
+        Generated observable list
     """
 
     assert len(means) == cov.shape[-1]
